%{
/* lexer for RNPL */
/* $Header: /home/felipe/rmarsa/source/rnpl/RCS/rnpl.l,v 2.2 1996/07/30 17:57:20 rmarsa Exp rmarsa $ */
/* Copyright (c) 1994-1996 by Robert Marsa */

#include "rnpl.h"
#include "y.tab.h"

BBH_I lineno=1;

%}

%p 3500

word ([a-zA-Z_][a-zA-Z_0-9]*)

%%

[ \t] ; /* ignore whitespace */

^#.* ; /* ignore comments (must start line) */

\/\/.* ; /* ignore comments (c++ like) */

(int|INT) {
  yylval.inum=INT;
  return INT;
}

(float|FLOAT) {
  yylval.inum=FLOAT;
  return FLOAT;
}

(string|STRING) {
  yylval.inum=STRING;
  return STRING;
}

(ivec|IVEC) {
	yylval.inum=IVEC;
	return IVEC;
}

(constant|CONSTANT) {
  yylval.inum=CONST;
  return CONST;
}

(system|SYSTEM) {
  yylval.inum=SYS;
  return SYS;
}

(parameter|PARAMETER) {
  yylval.inum=PARAM;
  return PARAM;
}

(attribute|ATTRIBUTE) {
  yylval.inum=ATTRIB;
  return ATTRIB;
}

(encodeone|ENCODEONE) {
  yylval.inum=ENCODEONE;
  return ENCODEONE;
}

(encodeall|ENCODEALL) {
  yylval.inum=ENCODEALL;
  return ENCODEALL;
}

(looper|LOOPER) {
  yylval.inum=LOOPER;
  return LOOPER;
}

(coordinates|COORDINATES) {
  yylval.inum=COORDINATES;
  return COORDINATES;
}

(grid|GRID) {
  yylval.inum=GRID;
  return GRID;
}

(on|ON) {
  yylval.inum=ON;
  return ON;
}

(at|AT) {
  yylval.inum=AT;
  return AT;
}

(alias|ALIAS) {
	yylval.inum=ALIAS;
	return ALIAS;
}

(expand|EXPAND) {
  yylval.inum=EXPAND;
  return EXPAND;
}

(operator|OPERATOR) {
  yylval.inum=OPERATOR;
  return OPERATOR;
}

(residual|RESIDUAL) {
  yylval.inum=RESID;
  return RESID;
}

(evaluate|EVALUATE) {
	yylval.inum=EVALUATE;
	return EVALUATE;
}

(auto|AUTO) {
	yylval.inum=AUTO;
	return AUTO;
}

(stub|STUB) {
	yylval.inum=STUB;
	return STUB;
}

(static|STATIC) {
	yylval.inum=STATIC;
	return STATIC;
}

(header|HEADER) {
	yylval.inum=HEADER;
	return HEADER;
}

(work|WORK) {
	yylval.inum=WORK;
	return WORK;
}

(if|IF) {
	yylval.inum=IF;
	return IF;
}

(then|THEN) {
	yylval.inum=THEN;
	return THEN;
}

(else|ELSE) {
	yylval.inum=ELSE;
	return ELSE;
}

(:=) {
  yylval.inum=ASSIGNOP;
  return ASSIGNOP;
}

(\") {
  yylval.inum=QUOTE;
  return QUOTE;
}

"+" {
  yylval.op='+';
  return PLUS;
}

"=" {
  yylval.op='=';
  return EQUALS;
}

"-" {
  yylval.op='-';
  return MINUS;
}

"*" {
  yylval.op='*';
  return TIMES;
}

\/ {
  yylval.op='/';
  return DIVIDE;
}

("^")|("**") {
  yylval.op='^';
  return CARET;
}

"==" {
	yylval.op='e';
	return EQUIV;
}

(\<) {
  yylval.op='<';
  return LESS;
}

(\>) {
  yylval.op='>';
  return GREAT;
}

"<=" {
	yylval.op='l';
	return LESSEQ;
}

">=" {
	yylval.op='g';
	return GREATEQ;
}

"!=" {
	yylval.op='n';
	return NOTEQ;
}

"||" {
	yylval.op='o';
	return OR;
}

"&&" {
	yylval.op='a';
	return AND;
}

"#" {
  yylval.inum=POUND;
  return POUND;
}

(\,) {
  yylval.inum=COMMA;
  return COMMA;
}

(\[) {
  yylval.inum=OBRACK;
  return OBRACK;
}

(\]) {
  yylval.inum=CBRACK;
  return CBRACK;
}

(\{) {
  yylval.inum=OBRACE;
  return OBRACE;
}

(\}) {
  yylval.inum=CBRACE;
  return CBRACE;
}

(\() {
  yylval.inum=OPAREN;
  return OPAREN;
}

(\)) {
  yylval.inum=CPAREN;
  return CPAREN;
}

(\:) {
  yylval.inum=COLON;
  return COLON;
}

(\;) {
  yylval.inum=SCOLON;
  return SCOLON;
}

(update|UPDATE|updates|UPDATES) {
  yylval.inum=UPDATE;
  return UPDATE;
}

(initialize|INITIALIZE) {
  yylval.inum=INITIALIZE;
  return INITIALIZE;
}

(uniform|UNIFORM) {
  yylval.inum=UNIFORM;
  return UNIFORM;
}

(nonuniform|NONUNIFORM) {
  yylval.inum=NONUNIFORM;
  return NONUNIFORM;
}

(\<[ ]*-?[0-9]+[ ]*\>) {
	yylval.inum = atoi((const char *)(yytext+1));
	return TIME;
}

([0-9]+) {
  yylval.inum = atoi((const char *)yytext);
  return INUM;
}

(([0-9]*\.[0-9]+)([eE][+-]?[0-9]+)?) {
  sscanf((const char *)yytext,"%lf",&yylval.num);
  return NUM;
}

{word}((\.{word})|("->"{word}))* {
  yylval.ident=(char *)malloc(strlen((const char *)yytext)+1);
  strcpy(yylval.ident,(const char *)yytext);
  return IDEN;
}

(\"[^\"\n]*\") {
  yylval.str = (char *)malloc(strlen((const char *)yytext)-1);
  strncpy(yylval.str,(const char *)yytext+1,strlen((const char *)yytext)-2);
  yylval.str[strlen((const char *)yytext)-2]=0;
  return STR;
}

\n { lineno++;}

%%

BBH_V yyerror(const BBH_C *msg)
{
  fprintf(stderr,"YYERROR:%d: %s at '%s'\n",lineno,msg,yytext);
	exit(1);
}

